@@ -193,6 +193,7 @@ class Image
 	}
 
 
+	/** @param  callable-string  $func */
 	private static function invokeSafe(string $func, string $arg, string $message, string $callee): static
 	{
 		$errors = [];
@@ -227,7 +228,7 @@ class Image
 		$image = new static(imagecreatetruecolor($width, $height));
 		if ($color) {
 			$image->alphaBlending(false);
-			$image->filledRectangle(0, 0, $width - 1, $height - 1, $color);
+			$image->filledRectangle(0, 0, $width - 1, $height - 1, self::normalizeColor($color));
 			$image->alphaBlending(true);
 		}
 
@@ -320,7 +321,7 @@ class Image
 	}
 
 
-	/** @return  ImageType[] */
+	/** @return  ImageType::*[] */
 	public static function getSupportedTypes(): array
 	{
 		self::ensureExtension();
@@ -392,6 +393,10 @@ class Image
 	public function resize(int|string|null $width, int|string|null $height, int $mode = self::OrSmaller): static
 	{
 		if ($mode & self::Cover) {
+			if ($width === null || $height === null) {
+				throw new Nette\InvalidArgumentException('Both width and height must be set for Cover mode.');
+			}
+
 			return $this->resize($width, $height, self::OrBigger)->crop('50%', '50%', $width, $height);
 		}
 
@@ -424,16 +429,14 @@ class Image
 
 	/**
 	 * Calculates dimensions of resized image. Width and height accept pixels or percent.
-	 * @param  int|string|null  $newWidth
-	 * @param  int|string|null  $newHeight
 	 * @param  int-mask-of<self::OrSmaller|self::OrBigger|self::Stretch|self::Cover|self::ShrinkOnly>  $mode
-	 * @return array{int, int}
+	 * @return array{int<1, max>, int<1, max>}
 	 */
 	public static function calculateSize(
 		int $srcWidth,
 		int $srcHeight,
-		$newWidth,
-		$newHeight,
+		int|string|null $newWidth,
+		int|string|null $newHeight,
 		int $mode = self::OrSmaller,
 	): array
 	{
@@ -477,19 +480,19 @@ class Image
 			}
 
 			if ($mode & self::OrBigger) {
-				$scale = [max($scale)];
+				$scale = [max($scale ?: [1])];
 			}
 
 			if ($mode & self::ShrinkOnly) {
 				$scale[] = 1;
 			}
 
-			$scale = min($scale);
+			$scale = min($scale ?: [1]);
 			$newWidth = (int) round($srcWidth * $scale);
 			$newHeight = (int) round($srcHeight * $scale);
 		}
 
-		return [max($newWidth, 1), max($newHeight, 1)];
+		return [max((int) $newWidth, 1), max((int) $newHeight, 1)];
 	}
 
 
@@ -504,7 +507,7 @@ class Image
 			$this->image = imagecrop($this->image, $r);
 			imagesavealpha($this->image, true);
 		} else {
-			$newImage = static::fromBlank($r['width'], $r['height'], ImageColor::rgb(0, 0, 0, 0))->getImageResource();
+			$newImage = static::fromBlank(max(1, $r['width']), max(1, $r['height']), ImageColor::rgb(0, 0, 0, 0))->getImageResource();
 			imagecopy($newImage, $this->image, 0, 0, $r['x'], $r['y'], $r['width'], $r['height']);
 			$this->image = $newImage;
 		}
@@ -526,21 +529,10 @@ class Image
 		int|string $newHeight,
 	): array
 	{
-		if (self::isPercent($newWidth)) {
-			$newWidth = (int) round($srcWidth / 100 * $newWidth);
-		}
-
-		if (self::isPercent($newHeight)) {
-			$newHeight = (int) round($srcHeight / 100 * $newHeight);
-		}
-
-		if (self::isPercent($left)) {
-			$left = (int) round(($srcWidth - $newWidth) / 100 * $left);
-		}
-
-		if (self::isPercent($top)) {
-			$top = (int) round(($srcHeight - $newHeight) / 100 * $top);
-		}
+		$newWidth = (int) (self::isPercent($newWidth) ? round($srcWidth / 100 * $newWidth) : $newWidth);
+		$newHeight = (int) (self::isPercent($newHeight) ? round($srcHeight / 100 * $newHeight) : $newHeight);
+		$left = (int) (self::isPercent($left) ? round(($srcWidth - $newWidth) / 100 * $left) : $left);
+		$top = (int) (self::isPercent($top) ? round(($srcHeight - $newHeight) / 100 * $top) : $top);
 
 		if ($left < 0) {
 			$newWidth += $left;
@@ -585,14 +577,8 @@ class Image
 
 		$width = $image->getWidth();
 		$height = $image->getHeight();
-
-		if (self::isPercent($left)) {
-			$left = (int) round(($this->getWidth() - $width) / 100 * $left);
-		}
-
-		if (self::isPercent($top)) {
-			$top = (int) round(($this->getHeight() - $height) / 100 * $top);
-		}
+		$left = (int) (self::isPercent($left) ? round(($this->getWidth() - $width) / 100 * $left) : $left);
+		$top = (int) (self::isPercent($top) ? round(($this->getHeight() - $height) / 100 * $top) : $top);
 
 		$output = $input = $image->image;
 		if ($opacity < 100) {
@@ -605,7 +591,7 @@ class Image
 			imagealphablending($output, false);
 			if (!$image->isTrueColor()) {
 				$input = $output;
-				imagefilledrectangle($output, 0, 0, $width, $height, imagecolorallocatealpha($output, 0, 0, 0, 127));
+				imagefilledrectangle($output, 0, 0, $width, $height, (int) imagecolorallocatealpha($output, 0, 0, 0, 127));
 				imagecopy($output, $image->image, 0, 0, 0, 0, $width, $height);
 			}
 
@@ -758,7 +744,7 @@ class Image
 
 	/**
 	 * Call to undefined method.
-	 * @param  array<mixed>  $args
+	 * @param  mixed[]  $args
 	 * @throws Nette\MemberAccessException
 	 */
 	public function __call(string $name, array $args): mixed
@@ -773,6 +759,7 @@ class Image
 				$args[$key] = $value->getImageResource();
 
 			} elseif ($value instanceof ImageColor || (is_array($value) && isset($value['red']))) {
+				/** @var ImageColor|array{red: int, green: int, blue: int, alpha?: int} $value */
 				$args[$key] = $this->resolveColor($value);
 			}
 		}
@@ -788,10 +775,11 @@ class Image
 	{
 		ob_start(fn() => '');
 		imagepng($this->image, null, 0);
-		$this->setImageResource(imagecreatefromstring(ob_get_clean()));
+		$this->setImageResource(imagecreatefromstring(ob_get_clean()) ?: throw new Nette\ShouldNotHappenException);
 	}
 
 
+	/** @param-out int|float $num */
 	private static function isPercent(int|string &$num): bool
 	{
 		if (is_string($num) && str_ends_with($num, '%')) {
@@ -820,11 +808,25 @@ class Image
 	 */
 	public function resolveColor(ImageColor|array $color): int
 	{
-		$color = $color instanceof ImageColor ? $color->toRGBA() : array_values($color + ['alpha' => 0]);
+		$color = self::normalizeColor($color)->toRGBA();
 		return imagecolorallocatealpha($this->image, ...$color) ?: imagecolorresolvealpha($this->image, ...$color);
 	}
 
 
+	/** @param  ImageColor|array{red: int, green: int, blue: int, alpha?: int}  $color */
+	private static function normalizeColor(ImageColor|array $color): ImageColor
+	{
+		return $color instanceof ImageColor
+			? $color
+			: ImageColor::rgb(
+				$color['red'],
+				$color['green'],
+				$color['blue'],
+				(127 - ($color['alpha'] ?? 0)) / 127,
+			);
+	}
+
+
 	private static function ensureExtension(): void
 	{
 		if (!extension_loaded('gd')) {
