@@ -10,7 +10,7 @@ declare(strict_types=1);
 namespace Nette\Utils;
 
 use Nette;
-use function array_map, array_search, array_splice, count, explode, implode, is_a, is_resource, is_string, strcasecmp, strtolower, substr, trim;
+use function array_map, array_search, array_splice, array_values, count, explode, implode, is_a, is_resource, is_string, strcasecmp, strtolower, substr, trim;
 
 
 /**
@@ -18,9 +18,9 @@ use function array_map, array_search, array_splice, count, explode, implode, is_
  */
 final readonly class Type
 {
-	/** @var array<int, string|self> */
+	/** @var list<string|self> */
 	private array $types;
-	private bool $simple;
+	private ?string $singleName;
 	private string $kind; // | &
 
 
@@ -40,8 +40,12 @@ final readonly class Type
 	}
 
 
-	/** @param  \ReflectionFunctionAbstract|\ReflectionParameter|\ReflectionProperty  $of */
-	private static function fromReflectionType(\ReflectionType $type, $of, bool $asObject): self|string
+	/** @return ($asObject is true ? self : self|string) */
+	private static function fromReflectionType(
+		\ReflectionType $type,
+		\ReflectionFunctionAbstract|\ReflectionParameter|\ReflectionProperty $of,
+		bool $asObject,
+	): self|string
 	{
 		if ($type instanceof \ReflectionNamedType) {
 			$name = self::resolve($type->getName(), $of);
@@ -114,12 +118,17 @@ final readonly class Type
 		$lower = strtolower($type);
 		if ($of instanceof \ReflectionFunction) {
 			return $type;
+		}
+
+		$class = $of->getDeclaringClass();
+		if ($class === null) {
+			return $type;
 		} elseif ($lower === 'self') {
-			return $of->getDeclaringClass()->name;
+			return $class->name;
 		} elseif ($lower === 'static') {
-			return ($of instanceof ReflectionMethod ? $of->getOriginalClass() : $of->getDeclaringClass())->name;
-		} elseif ($lower === 'parent' && $of->getDeclaringClass()->getParentClass()) {
-			return $of->getDeclaringClass()->getParentClass()->name;
+			return ($of instanceof ReflectionMethod ? $of->getOriginalClass() : $class)->name;
+		} elseif ($lower === 'parent' && $class->getParentClass()) {
+			return $class->getParentClass()->name;
 		} else {
 			return $type;
 		}
@@ -131,12 +140,12 @@ final readonly class Type
 	{
 		$o = array_search('null', $types, strict: true);
 		if ($o !== false) { // null as last
-			array_splice($types, $o, 1);
+			array_splice($types, (int) $o, 1);
 			$types[] = 'null';
 		}
 
-		$this->types = $types;
-		$this->simple = is_string($types[0]) && ($types[1] ?? 'null') === 'null';
+		$this->types = array_values($types);
+		$this->singleName = is_string($types[0]) && ($types[1] ?? 'null') === 'null' ? $types[0] : null;
 		$this->kind = count($types) > 1 ? $kind : '';
 	}
 
@@ -144,8 +153,8 @@ final readonly class Type
 	public function __toString(): string
 	{
 		$multi = count($this->types) > 1;
-		if ($this->simple) {
-			return ($multi ? '?' : '') . $this->types[0];
+		if ($this->singleName !== null) {
+			return ($multi ? '?' : '') . $this->singleName;
 		}
 
 		$res = [];
@@ -184,8 +193,8 @@ final readonly class Type
 
 
 	/**
-	 * Returns the array of subtypes that make up the compound type as Type objects:
-	 * @return self[]
+	 * Returns the array of subtypes that make up the compound type as Type objects.
+	 * @return list<self>
 	 */
 	public function getTypes(): array
 	{
@@ -198,9 +207,7 @@ final readonly class Type
 	 */
 	public function getSingleName(): ?string
 	{
-		return $this->simple
-			? $this->types[0]
-			: null;
+		return $this->singleName;
 	}
 
 
@@ -227,14 +234,14 @@ final readonly class Type
 	 */
 	public function isSimple(): bool
 	{
-		return $this->simple;
+		return $this->singleName !== null;
 	}
 
 
 	#[\Deprecated('use isSimple()')]
 	public function isSingle(): bool
 	{
-		return $this->simple;
+		return $this->singleName !== null;
 	}
 
 
@@ -243,7 +250,7 @@ final readonly class Type
 	 */
 	public function isBuiltin(): bool
 	{
-		return $this->simple && Validators::isBuiltinType($this->types[0]);
+		return $this->singleName !== null && Validators::isBuiltinType($this->singleName);
 	}
 
 
@@ -252,7 +259,7 @@ final readonly class Type
 	 */
 	public function isClass(): bool
 	{
-		return $this->simple && !Validators::isBuiltinType($this->types[0]);
+		return $this->singleName !== null && !Validators::isBuiltinType($this->singleName);
 	}
 
 
@@ -261,7 +268,7 @@ final readonly class Type
 	 */
 	public function isClassKeyword(): bool
 	{
-		return $this->simple && Validators::isClassKeyword($this->types[0]);
+		return $this->singleName !== null && Validators::isClassKeyword($this->singleName);
 	}
 
 
@@ -281,7 +288,7 @@ final readonly class Type
 	}
 
 
-	/** @param  (string|self)[]  $givenTypes */
+	/** @param array<string>  $givenTypes */
 	private function allowsAny(array $givenTypes): bool
 	{
 		return $this->isUnion()
@@ -291,16 +298,16 @@ final readonly class Type
 
 
 	/**
-	 * @param  (string|self)[]  $ourTypes
-	 * @param  (string|self)[]  $givenTypes
+	 * @param array<string>  $ourTypes
+	 * @param array<string>  $givenTypes
 	 */
 	private function allowsAll(array $ourTypes, array $givenTypes): bool
 	{
 		return Arrays::every(
 			$ourTypes,
-			fn($ourType) => Arrays::some(
+			fn(string $ourType) => Arrays::some(
 				$givenTypes,
-				fn($givenType) => Validators::isBuiltinType($ourType)
+				fn(string $givenType) => Validators::isBuiltinType($ourType)
 					? strcasecmp($ourType, $givenType) === 0
 					: is_a($givenType, $ourType, allow_string: true),
 			),
